# Generated by Django 3.2 on 2021-04-14 20:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0065_emailtemplate_mail_to'),
    ]

    operations = [
        migrations.RenameField(
            model_name='bestellung',
            old_name='rechnungsemail',
            new_name='email_rechnung',
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='email_rechnung',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='kmuhelper.email'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='email_lieferung',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='kmuhelper.email'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='lieferadresse_email',
            field=models.EmailField(blank=True, max_length=254, verbose_name='E-Mail Adresse'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='lieferadresse_telefon',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name='Telefon'),
        ),
        migrations.AddField(
            model_name='kunde',
            name='lieferadresse_email',
            field=models.EmailField(blank=True, max_length=254, verbose_name='E-Mail Adresse'),
        ),
        migrations.AddField(
            model_name='kunde',
            name='lieferadresse_telefon',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name='Telefon'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-05-02 13:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0006_auto_20200428_1711'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellung',
            name='rechnung_gesendet',
            field=models.BooleanField(default=False, verbose_name='Rechnung gesendet'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungsmethode',
            field=models.CharField(choices=[('bacs', 'Überweisung'), ('cheque', 'Scheck'), ('cod', 'Rechnung / Nachnahme'), ('paypal', 'PayPal')], default='cod', max_length=7, verbose_name='Zahlungsmethode'),
        ),
    ]



# Generated by Django 3.1 on 2020-10-06 16:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0038_produkt_soll_lagerbestand'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='bezahlt',
            field=models.BooleanField(default=False, help_text='Sobald eine Bestellung als bezahlt markiert wurde, können Teile der Bestellung nicht mehr bearbeitet werden!', verbose_name='Bezahlt'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='versendet',
            field=models.BooleanField(default=False, help_text='Sobald eine Bestellung als versendet markiert wurde, können Teile der Bestellung nicht mehr bearbeitet werden! Ausserdem werden die Produkte aus dem Lagerbestand entfernt.', verbose_name='Versendet'),
        ),
    ]



from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.urls import reverse_lazy, reverse
from django.shortcuts import render, redirect

from kmuhelper.decorators import require_object
from kmuhelper.integrations.paymentimport.forms import CamtUploadForm
from kmuhelper.integrations.paymentimport.models import PaymentImport


@login_required(login_url=reverse_lazy('admin:login'))
@permission_required('kmuhelper.add_paymentimport')
def upload(request):
    if request.method == 'POST':
        form = CamtUploadForm(request.POST, request.FILES)
        if form.is_valid():
            return form.create_payment_import_and_redirect(request)
    else:
        form = CamtUploadForm()

    return render(request, 'admin/kmuhelper/paymentimport/upload.html', {
        'form': form,
        'has_permission': True
    })


@login_required(login_url=reverse_lazy('admin:login'))
@permission_required('kmuhelper.change_paymentimport')
@require_object(PaymentImport)
def process(request, obj):
    if request.method == 'POST':
        obj.is_processed = True
        obj.save()
        messages.success(request, "Zahlungsimport wurde als verarbeitet markiert!")
        return redirect(reverse('admin:kmuhelper_paymentimport_changelist'))
    
    if obj.is_processed:
        messages.warning(request, "Achtung! Dieser Zahlungsimport wurde bereits als verarbeitet markiert!")

    ctx = obj.get_processing_context()
    return render(request, 'admin/kmuhelper/paymentimport/process.html', {
        'has_permission': True,
        'original': obj,
        'data': ctx,
    })



from django.db import models
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt

from kmuhelper.utils import package_version, python_version
from kmuhelper.main.models import Bestellung
from kmuhelper.api.constants import ENDPOINT_NOT_FOUND, SUCCESSFULLY_CHANGED
from kmuhelper.api.decorators import require_object, api_read, api_write, api_readwrite

#####


def not_found(request):
    """Return a json error message"""

    return ENDPOINT_NOT_FOUND

#####


@api_read()
def versions(request):
    """Get the installed versions of some dependencies, their newest versions available and the current python version."""

    context = {
        "versions": {
            "Python": {"current": python_version(), "latest": "-", "uptodate": None},
        }
    }

    packages = ["Django", "django-kmuhelper", "gunicorn",
                "requests", "WooCommerce", "reportlab", "rich", "pytz"]

    for package in packages:
        context["versions"][package] = package_version(package)
    return JsonResponse(context)


@api_read()
def orders_unpaid(request):
    """Get sums of currently unpaid orders."""

    sum_unsent = Bestellung.objects.filter(bezahlt=False, versendet=False).aggregate(
        models.Sum('fix_summe'))["fix_summe__sum"] or 0
    sum_sent = Bestellung.objects.filter(bezahlt=False, versendet=True).aggregate(
        models.Sum('fix_summe'))["fix_summe__sum"] or 0

    context = {
        "orders_unpaid_sum": {
            "unsent": round(sum_unsent, 2),
            "sent": round(sum_sent, 2),
            "all": round(sum_unsent, 2)+round(sum_sent, 2)
        }
    }
    return JsonResponse(context)


@csrf_exempt
@api_write(['kmuhelper.change_order'])
@require_object(Bestellung)
def orders_set_paid(request, obj):
    """Set an order as paid"""

    if not obj.bezahlt:
        obj.bezahlt = True
        if obj.versendet and obj.status == 'pending':
            obj.status = 'completed'
        obj.save()
    return SUCCESSFULLY_CHANGED



# Generated by Django 3.0.4 on 2020-07-20 16:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0027_auto_20200717_1630'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='bestellung',
            name='notiz',
        ),
        migrations.AddField(
            model_name='notiz',
            name='bestellung',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notiz', to='kmuhelper.Bestellung'),
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-03 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0051_apikey'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellung',
            name='rechnungsdatum',
            field=models.DateField(blank=True, default=None, null=True, verbose_name='Rechnungsdatum'),
        ),
    ]



from django.utils.html import strip_tags

from rich import print
from rich.progress import Progress

from woocommerce import API as WCAPI

from kmuhelper import settings
from kmuhelper.main.models import Produkt, Kunde, Kategorie, Bestellung, Kosten
from kmuhelper.utils import runden


PREFIX = "[deep_pink4][KMUHelper][/] -"


def log(string, *args):
    print(PREFIX, string, *args)


def preparestring(string):
    """Prepare a HTML string for import"""
    return strip_tags(string.replace("</p>", " ").replace("</strong>", " ")).replace("&#8211;", "-").replace("&#8211;", " ").replace("&#215;", "x").replace("&#8220;", '"').replace("&#8221;", '"').replace("&nbsp;", " ").replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&").replace("&quot;", '"').replace("&#8222;", '"').replace("  ", " ")

###############


class WooCommerce():
    """Manage connection with WooCommerce"""

    @classmethod
    def get_api(cls):
        """Create a API object from data stored in settings"""
        return WCAPI(
            url=settings.get_secret_db_setting(
                "wc-url"),
            consumer_key=settings.get_secret_db_setting(
                "wc-consumer_key"),
            consumer_secret=settings.get_secret_db_setting(
                "wc-consumer_secret")
        )

    # General methods

    @classmethod
    def product_create(cls, product, api=None):
        """Create a new product from WooCommerce data"""
        wcapi = api or cls.get_api()

        try:
            verkaufspreis = float(product["price"])
        except ValueError:
            verkaufspreis = 0.0

        newproduct = Produkt.objects.create(
            woocommerceid=product["id"],
            artikelnummer=product["sku"],
            name=preparestring(product["name"]),
            kurzbeschrieb=preparestring(product["short_description"]),
            beschrieb=preparestring(product["description"]),
            verkaufspreis=verkaufspreis,
            bildlink=(product["images"][0]["src"]) if len(
                product["images"]) > 0 else "",
            aktion_von=(product["date_on_sale_from_gmt"] +
                        "+00:00" if product["date_on_sale_from"] else None),
            aktion_bis=(product["date_on_sale_to_gmt"] +
                        "+00:00" if product["date_on_sale_to"] else None),
            aktion_preis=(product["sale_price"]
                          if product["sale_price"] else None)
        )
        if product["manage_stock"]:
            newproduct.lagerbestand = product["stock_quantity"]
        for category in product["categories"]:
            tup = Kategorie.objects.get_or_create(woocommerceid=category["id"])
            if tup[1]:
                cls.category_update(tup[0], api=wcapi)
            newproduct.kategorien.add(tup[0])
        newproduct.save()
        log("Produkt erstellt:", str(newproduct))
        return newproduct

    @classmethod
    def product_update(cls, product, newproduct=None, api=None):
        """Update a specific product from WooCommerce"""
        wcapi = api or cls.get_api()

        if not newproduct:
            newproduct = wcapi.get(f'products/{product.woocommerceid}').json()

        try:
            product.verkaufspreis = float(newproduct["price"])
        except ValueError:
            pass
        except KeyError:
            if "code" in newproduct and newproduct["code"] == "woocommerce_rest_product_invalid_id":
                log(PREFIX +
                    " [red]Produkt existiert in WooCommerce nicht![/]")
                product.woocommerceid = 0
                product.save()
                return product

        product.artikelnummer = newproduct["sku"]
        product.name = preparestring(newproduct["name"])
        product.kurzbeschrieb = preparestring(newproduct["short_description"])
        product.beschrieb = preparestring(newproduct["description"])
        product.bildlink = (newproduct["images"][0]["src"]) if len(
            newproduct["images"]) > 0 else ""
        if newproduct["date_on_sale_from_gmt"]:
            product.aktion_von = newproduct["date_on_sale_from_gmt"] + "+00:00"
        if newproduct["date_on_sale_to_gmt"]:
            product.aktion_bis = newproduct["date_on_sale_to_gmt"] + "+00:00"
        if newproduct["sale_price"]:
            product.aktion_preis = newproduct["sale_price"]

        product.kategorien.clear()
        newcategories = []
        for category in newproduct["categories"]:
            tup = Kategorie.objects.get_or_create(woocommerceid=category["id"])
            if tup[1]:
                cls.category_update(tup[0], api=wcapi)
            newcategories.append(tup[0])
        product.kategorien.add(*newcategories)

        product.save()
        log("Produkt aktualisiert:", str(product))
        return product

    @classmethod
    def product_import(cls, api=None):
        """Import new products from WooCommerce"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task_prepare = progress.add_task(
                PREFIX + " [orange_red1]Produktdownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Produkt.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            productlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                PREFIX + " [green]Produkte herunterladen...")

            r = wcapi.get("products?exclude=" + excludeids)
            productlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                productlist += wcapi.get("products?exclude=" +
                                         excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)

            progress.stop_task(task_download)

            if productlist:
                task_process = progress.add_task(
                    PREFIX + " [cyan]Produkte verarbeiten...", total=len(productlist))
                for product in productlist:
                    cls.product_create(product, api=wcapi)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(productlist)

    @classmethod
    def product_bulk_update(cls, products, api=None):
        """Update every product in a queryset"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task = progress.add_task(
                PREFIX + " [orange_red1]Produkte aktualisieren...", total=products.count())
            successcount = 0
            errorcount = 0
            for product in products:
                if product.woocommerceid:
                    cls.product_update(product, api=wcapi)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def customer_create(cls, customer):
        """Create a new customer from WooCommerce data"""
        newcustomer = Kunde.objects.create(
            woocommerceid=customer["id"],

            email=customer["email"],
            vorname=customer["first_name"],
            nachname=customer["last_name"],
            firma=customer["billing"]["company"],
            benutzername=customer["username"],
            avatar_url=customer["avatar_url"],

            rechnungsadresse_vorname=customer["billing"]["first_name"],
            rechnungsadresse_nachname=customer["billing"]["last_name"],
            rechnungsadresse_firma=customer["billing"]["company"],
            rechnungsadresse_adresszeile1=customer["billing"]["address_1"],
            rechnungsadresse_adresszeile2=customer["billing"]["address_2"],
            rechnungsadresse_ort=customer["billing"]["city"],
            rechnungsadresse_kanton=customer["billing"]["state"],
            rechnungsadresse_plz=customer["billing"]["postcode"],
            rechnungsadresse_land=customer["billing"]["country"],
            rechnungsadresse_email=customer["billing"]["email"],
            rechnungsadresse_telefon=customer["billing"]["phone"],

            lieferadresse_vorname=customer["shipping"]["first_name"],
            lieferadresse_nachname=customer["shipping"]["last_name"],
            lieferadresse_firma=customer["shipping"]["company"],
            lieferadresse_adresszeile1=customer["shipping"]["address_1"],
            lieferadresse_adresszeile2=customer["shipping"]["address_2"],
            lieferadresse_ort=customer["shipping"]["city"],
            lieferadresse_kanton=customer["shipping"]["state"],
            lieferadresse_plz=customer["shipping"]["postcode"],
            lieferadresse_land=customer["shipping"]["country"],
            lieferadresse_email="",
            lieferadresse_telefon="",
        )
        log("Kunde erstellt:", str(newcustomer))
        return newcustomer

    @classmethod
    def customer_update(cls, customer, newcustomer=None, api=None):
        """Update a specific customer from WooCommerce"""
        wcapi = api or cls.get_api()

        if not newcustomer:
            newcustomer = wcapi.get(f'customers/{customer.woocommerceid}').json()

        if "code" in newcustomer and newcustomer["code"] == "woocommerce_rest_customer_invalid_id":
            log("[red]Kunde existiert in WooCommerce nicht![/]")
            customer.woocommerceid = 0
            customer.save()
            return customer

        customer.email = newcustomer["email"]
        customer.vorname = newcustomer["first_name"]
        customer.nachname = newcustomer["last_name"]
        customer.firma = newcustomer["billing"]["company"] if newcustomer["billing"]["company"] else customer.firma
        customer.benutzername = newcustomer["username"]
        customer.avatar_url = newcustomer["avatar_url"]

        customer.rechnungsadresse_vorname = newcustomer["billing"]["first_name"]
        customer.rechnungsadresse_nachname = newcustomer["billing"]["last_name"]
        customer.rechnungsadresse_firma = newcustomer["billing"]["company"]
        customer.rechnungsadresse_adresszeile1 = newcustomer["billing"]["address_1"]
        customer.rechnungsadresse_adresszeile2 = newcustomer["billing"]["address_2"]
        customer.rechnungsadresse_ort = newcustomer["billing"]["city"]
        customer.rechnungsadresse_kanton = newcustomer["billing"]["state"]
        customer.rechnungsadresse_plz = newcustomer["billing"]["postcode"]
        customer.rechnungsadresse_land = newcustomer["billing"]["country"]
        customer.rechnungsadresse_email = newcustomer["billing"]["email"]
        customer.rechnungsadresse_telefon = newcustomer["billing"]["phone"]

        customer.lieferadresse_vorname = newcustomer["shipping"]["first_name"]
        customer.lieferadresse_nachname = newcustomer["shipping"]["last_name"]
        customer.lieferadresse_firma = newcustomer["shipping"]["company"]
        customer.lieferadresse_adresszeile1 = newcustomer["shipping"]["address_1"]
        customer.lieferadresse_adresszeile2 = newcustomer["shipping"]["address_2"]
        customer.lieferadresse_ort = newcustomer["shipping"]["city"]
        customer.lieferadresse_kanton = newcustomer["shipping"]["state"]
        customer.lieferadresse_plz = newcustomer["shipping"]["postcode"]
        customer.lieferadresse_land = newcustomer["shipping"]["country"]
        customer.save()
        log("Kunde aktualisiert:", str(customer))
        return customer

    @classmethod
    def customer_import(cls, api=None):
        """Import new customers from WooCommerce"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task_prepare = progress.add_task(
                PREFIX + " [orange_red1]Kundendownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Kunde.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            customerlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                PREFIX + " [green]Kunden herunterladen...")

            r = wcapi.get("customers?exclude=" + excludeids)
            customerlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                customerlist += wcapi.get("customers?exclude=" +
                                          excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)
            progress.stop_task(task_download)

            if customerlist:
                task_process = progress.add_task(
                    PREFIX + " [cyan]Kunden verarbeiten...", total=len(customerlist))
                for customer in customerlist:
                    cls.customer_create(customer)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(customerlist)

    @classmethod
    def customer_bulk_update(cls, customers, api=None):
        """Update every customer in a queryset"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task = progress.add_task(
                PREFIX + " [orange_red1]Kunden aktualisieren...", total=customers.count())
            successcount = 0
            errorcount = 0
            for customer in customers:
                if customer.woocommerceid:
                    cls.customer_update(customer, api=wcapi)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def category_update(cls, category, newcategory=None, api=None):
        """Update a specific category from WooCommerce"""
        wcapi = api or cls.get_api()

        if not newcategory:
            newcategory = wcapi.get(f'products/categories/{category.woocommerceid}').json()

        if "code" in newcategory and newcategory["code"] == "woocommerce_rest_category_invalid_id":
            log("[red]Kategorie existiert in WooCommerce nicht![/]")
            category.woocommerceid = 0
            category.save()
            return category

        category.name = preparestring(newcategory["name"])
        category.beschrieb = preparestring(newcategory["description"])
        category.bildlink = (
            newcategory["image"]["src"]) if newcategory["image"] else ""
        if newcategory["parent"]:
            tup = Kategorie.objects.get_or_create(
                woocommerceid=newcategory["parent"])
            if tup[1]:
                cls.category_update(tup[0], api=wcapi)
            category.uebergeordnete_kategorie = tup[0]
        category.save()
        log("Kategorie aktualisiert:", str(category))
        return category

    @classmethod
    def category_import(cls, api=None):
        """Import new categories from WooCommerce"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task_prepare = progress.add_task(
                PREFIX + " [orange_red1]Kategoriendownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Kategorie.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            categorylist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                PREFIX + " [green]Kategorien herunterladen...")

            r = wcapi.get("products/categories?exclude=" + excludeids)
            categorylist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                categorylist += wcapi.get("products/categories?exclude=" +
                                          excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)

            progress.stop_task(task_download)

            categorieswithparents = []

            if categorylist:
                task_process = progress.add_task(
                    PREFIX + " [cyan]Kategorien verarbeiten...", total=len(categorylist))
                for category in categorylist:
                    newcategory = Kategorie.objects.create(
                        name=preparestring(category["name"]),
                        beschrieb=preparestring(category["description"]),
                        bildlink=(category["image"]["src"]
                                  ) if category["image"] else "",
                        woocommerceid=category["id"]
                    )
                    log("Kategorie erstellt:", category["name"])
                    if category["parent"]:
                        categorieswithparents.append(
                            (newcategory, category["parent"]))
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)

            if categorieswithparents:
                task_process2 = progress.add_task(
                    PREFIX + " [cyan]Kategorieabhängigkeiten verarbeiten...", total=len(categorieswithparents))
                for tup in categorieswithparents:
                    tup[0].uebergeordnete_kategorie = Kategorie.objects.get(
                        woocommerceid=tup[1])
                    tup[0].save()
                    progress.update(task_process2, advance=1)
                progress.stop_task(task_process2)
        return len(categorylist)

    @classmethod
    def category_bulk_update(cls, categories, api=None):
        """Update every category in a queryset"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task = progress.add_task(
                PREFIX + " [orange_red1]Kategorien aktualisieren...", total=categories.count())
            successcount = 0
            errorcount = 0
            for category in categories:
                if category.woocommerceid:
                    cls.category_update(category, api=wcapi)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)

    @classmethod
    def order_create(cls, order, api=None, sendstockwarning=True):
        """Create a new order from WooCommerce data"""
        wcapi = api or cls.get_api()

        neworder = Bestellung.objects.create(
            woocommerceid=order["id"],

            status=order["status"],
            versendet=(True if order["status"] == "completed" else False),

            ausgelagert=(True if order["status"] == "completed" else False),

            zahlungsmethode=order["payment_method"],
            bezahlt=(True if order["date_paid"] else False),

            kundennotiz=order["customer_note"],

            order_key=order["order_key"],

            rechnungsadresse_vorname=order["billing"]["first_name"],
            rechnungsadresse_nachname=order["billing"]["last_name"],
            rechnungsadresse_firma=order["billing"]["company"],
            rechnungsadresse_adresszeile1=order["billing"]["address_1"],
            rechnungsadresse_adresszeile2=order["billing"]["address_2"],
            rechnungsadresse_ort=order["billing"]["city"],
            rechnungsadresse_kanton=order["billing"]["state"],
            rechnungsadresse_plz=order["billing"]["postcode"],
            rechnungsadresse_land=order["billing"]["country"],
            rechnungsadresse_email=order["billing"]["email"],
            rechnungsadresse_telefon=order["billing"]["phone"],

            lieferadresse_vorname=order["shipping"]["first_name"],
            lieferadresse_nachname=order["shipping"]["last_name"],
            lieferadresse_firma=order["shipping"]["company"],
            lieferadresse_adresszeile1=order["shipping"]["address_1"],
            lieferadresse_adresszeile2=order["shipping"]["address_2"],
            lieferadresse_ort=order["shipping"]["city"],
            lieferadresse_kanton=order["shipping"]["state"],
            lieferadresse_plz=order["shipping"]["postcode"],
            lieferadresse_land=order["shipping"]["country"]
        )
        neworder.datum = order["date_created_gmt"] + "+00:00"
        if order["customer_id"]:
            kunde = Kunde.objects.get_or_create(
                woocommerceid=int(order["customer_id"]))
            if kunde[1]:
                kunde = cls.customer_update(kunde[0], api=wcapi)
            else:
                kunde = kunde[0]

            neworder.kunde = kunde
            neworder.lieferadresse_email = kunde.lieferadresse_email
            neworder.lieferadresse_telefon = kunde.lieferadresse_telefon

        for item in order["line_items"]:
            product = Produkt.objects.get_or_create(
                woocommerceid=int(item["product_id"]))
            if product[1]:
                product = cls.product_update(product[0], api=wcapi)
            else:
                product = product[0]

            neworder.produkte.add(
                product, 
                through_defaults={
                    "menge": int(item["quantity"]),
                    "produktpreis": runden(float(item["price"]))
                }
            )
        for item in order["shipping_lines"]:
            price = float(item["total"])

            neworder.kosten.add(
                Kosten.objects.get_or_create(
                    name=item["method_title"],
                    preis=price,
                    mwstsatz=(7.7 if float(item["total_tax"]) > 0 else 0)
                )[0],
                through_defaults={"kostenpreis": price}
            )
        neworder.save()
        neworder.second_save()
        log("Bestellung erstellt:", str(neworder))

        if sendstockwarning:
            neworder.email_stock_warning()
        return neworder

    @classmethod
    def order_update(cls, order, neworder=None, api=None):
        """Update a specific order from WooCommerce"""
        wcapi = api or cls.get_api()

        if not neworder:
            neworder = wcapi.get(f'orders/{order.woocommerceid}').json()

        if "code" in neworder and neworder["code"] == "woocommerce_rest_order_invalid_id":
            log("[red]Bestellung existiert in WooCommerce nicht![/]")
            order.woocommerceid = 0
            order.save()
            return order

        if neworder["date_paid"]:
            order.bezahlt = True
        order.status = neworder["status"]
        order.kundennotiz = neworder["customer_note"]

        order.rechnungsadresse_vorname = neworder["billing"]["first_name"]
        order.rechnungsadresse_nachname = neworder["billing"]["last_name"]
        order.rechnungsadresse_firma = neworder["billing"]["company"]
        order.rechnungsadresse_adresszeile1 = neworder["billing"]["address_1"]
        order.rechnungsadresse_adresszeile2 = neworder["billing"]["address_2"]
        order.rechnungsadresse_ort = neworder["billing"]["city"]
        order.rechnungsadresse_kanton = neworder["billing"]["state"]
        order.rechnungsadresse_plz = neworder["billing"]["postcode"]
        order.rechnungsadresse_land = neworder["billing"]["country"]
        order.rechnungsadresse_email = neworder["billing"]["email"]
        order.rechnungsadresse_telefon = neworder["billing"]["phone"]

        order.lieferadresse_vorname = neworder["shipping"]["first_name"]
        order.lieferadresse_nachname = neworder["shipping"]["last_name"]
        order.lieferadresse_firma = neworder["shipping"]["company"]
        order.lieferadresse_adresszeile1 = neworder["shipping"]["address_1"]
        order.lieferadresse_adresszeile2 = neworder["shipping"]["address_2"]
        order.lieferadresse_ort = neworder["shipping"]["city"]
        order.lieferadresse_kanton = neworder["shipping"]["state"]
        order.lieferadresse_plz = neworder["shipping"]["postcode"]
        order.lieferadresse_land = neworder["shipping"]["country"]
        order.save()
        log("Bestellung aktualisiert: ", str(order))
        return order

    @classmethod
    def order_import(cls, api=None):
        """Import new orders from WooCommerce"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task_prepare = progress.add_task(
                PREFIX + " [orange_red1]Bestellungsdownload vorbereiten...", total=1)

            excludeids = str([obj.woocommerceid for obj in Bestellung.objects.all().exclude(
                woocommerceid=0)])[1:-1]
            orderlist = []

            progress.update(task_prepare, advance=1)
            progress.stop_task(task_prepare)

            task_download = progress.add_task(
                PREFIX + " [green]Bestellungen herunterladen...")

            r = wcapi.get("orders?exclude=" + excludeids)
            orderlist += r.json()

            progress.update(task_download, advance=1,
                            total=int(r.headers['X-WP-TotalPages']))

            for page in range(2, int(r.headers['X-WP-TotalPages']) + 1):
                orderlist += wcapi.get("orders?exclude=" +
                                       excludeids + "&page=" + str(page)).json()
                progress.update(task_download, advance=1)
            progress.stop_task(task_download)

            if orderlist:
                task_process = progress.add_task(
                    PREFIX + " [cyan]Bestellungen verarbeiten...", total=len(orderlist))
                for order in orderlist:
                    cls.order_create(order, api=wcapi, sendstockwarning=False)
                    progress.update(task_process, advance=1)
                progress.stop_task(task_process)
        return len(orderlist)

    @classmethod
    def order_bulk_update(cls, orders, api=None):
        """Update every order in a queryset"""
        wcapi = api or cls.get_api()

        with Progress() as progress:
            task = progress.add_task(
                PREFIX + " [orange_red1]Bestellungen aktualisieren...", total=orders.count())
            successcount = 0
            errorcount = 0
            for order in orders:
                if order.woocommerceid:
                    cls.order_update(order, api=wcapi)
                    successcount += 1
                else:
                    errorcount += 1
                progress.update(task, advance=1)
            progress.stop_task(task)
        return (successcount, errorcount)



# Generated by Django 3.0.4 on 2020-08-02 19:05

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0032_auto_20200802_0018'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ansprechpartner',
            name='email',
            field=models.EmailField(help_text='Auf Rechnung ersichtlich!', max_length=254, verbose_name='E-Mail'),
        ),
        migrations.AlterField(
            model_name='ansprechpartner',
            name='name',
            field=models.CharField(help_text='Auf Rechnung ersichtlich!', max_length=50, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='ansprechpartner',
            name='telefon',
            field=models.CharField(help_text='Auf Rechnung ersichtlich!', max_length=50, verbose_name='Telefon'),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='adresszeile1',
            field=models.CharField(max_length=70, verbose_name="Strasse und Hausnummer oder 'Postfach'"),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='firmenname',
            field=models.CharField(help_text='Name der Firma', max_length=70, verbose_name='Firmennname'),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='firmenuid',
            field=models.CharField(help_text='UID der Firma - Format: CHE-123.456.789 (Mehrwertsteuernummer)', max_length=15, validators=[django.core.validators.RegexValidator('^CHE-[0-9]{3}\\.[0-9]{3}\\.[0-9]{3}$', 'Bite benutze folgendes Format: CHE-123.456.789')], verbose_name='Firmen-UID'),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='logourl',
            field=models.URLField(help_text='URL eines Bildes (.jpg/.png) - Wird auf die Rechnung gedruckt.', validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z\\-\\.\\|\\?\\(\\)\\*\\+&"\'_:;/]+\\.(png|jpg)$', 'Nur folgende Zeichen gestattet: 0-9a-zA-Z-_.:;/|?&()"\'*+ - Muss auf .jpg/.png enden.')], verbose_name='Logo (URL)'),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='qriban',
            field=models.CharField(help_text='QR-IBAN mit Leerzeichen', max_length=26, validators=[django.core.validators.RegexValidator('^CH[0-9]{2}\\s3[0-9]{3}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{1}$', 'Bite benutze folgendes Format: CHxx 3xxx xxxx xxxx xxxx x')], verbose_name='QR-IBAN'),
        ),
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='webseite',
            field=models.URLField(help_text='Auf der Rechnung ersichtlich!', verbose_name='Webseite'),
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-03 15:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0052_bestellung_rechnungsdatum'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellung',
            name='rechnungstext',
            field=models.TextField(blank=True, default='', help_text="Wird auf der Rechnung gedruckt! Unterstützt <abbr title='<b>Fett</b><u>Unterstrichen</u><i>Kursiv</i>'>bestimmten XML markup</abbr>.", verbose_name='Rechnungstext'),
        ),
    ]



from datetime import datetime
from pytz import utc

from django.contrib import admin, messages
from django.urls import path

from kmuhelper import constants
from kmuhelper.integrations.woocommerce import WooCommerce
from kmuhelper.main import views
from kmuhelper.main.models import (
    Ansprechpartner, Bestellung, Kategorie, Kosten, Kunde,
    Lieferant, Lieferung, Notiz, Produkt, Zahlungsempfaenger, Einstellung
)
from kmuhelper.overrides import CustomModelAdmin, CustomTabularInline, CustomStackedInline


#######


@admin.register(Ansprechpartner)
class AnsprechpartnerAdmin(CustomModelAdmin):
    fieldsets = [
        ("Name", {'fields': ['name']}),
        ('Daten', {'fields': ['telefon', 'email']})
    ]

    ordering = ('name',)

    list_display = ('name', 'telefon', 'email')
    search_fields = ['name', 'telefon', 'email']


class BestellungInlineBestellungsposten(CustomTabularInline):
    model = Bestellung.produkte.through
    verbose_name = "Bestellungsposten"
    verbose_name_plural = "Bestellungsposten"
    extra = 0

    fieldsets = [
        (None, {'fields': ['produkt', 'bemerkung', 'produktpreis',
                           'menge', 'rabatt', 'mwstsatz', 'zwischensumme']})
    ]

    readonly_fields = ('zwischensumme', 'mwstsatz', 'produkt',)

    def get_additional_readonly_fields(self, request, obj=None):
        fields = ["produktpreis"]
        if obj and (obj.versendet or obj.bezahlt):
            fields += ["menge"]
        if obj and obj.bezahlt:
            fields += ["rabatt"]
        return fields

    # Permissions

    NO_ADD = True

    def has_delete_permission(self, request, obj=None):
        return False if (obj and (obj.versendet or obj.bezahlt)) else super().has_delete_permission(request, obj)


class BestellungInlineBestellungspostenAdd(CustomTabularInline):
    model = Bestellung.produkte.through
    verbose_name = "Bestellungsposten"
    verbose_name_plural = "Bestellungsposten hinzufügen"
    extra = 0

    autocomplete_fields = ("produkt",)

    fieldsets = [
        (None, {'fields': ['produkt', 'bemerkung', 'menge', 'rabatt']})
    ]

    # Permissions

    NO_CHANGE = True
    NO_DELETE = True
    NO_VIEW = True

    def has_add_permission(self, request, obj=None):
        return False if (obj and (obj.versendet or obj.bezahlt)) else super().has_add_permission(request, obj)


class BestellungInlineBestellungskosten(CustomTabularInline):
    model = Bestellung.kosten.through
    verbose_name = "Bestellungskosten"
    verbose_name_plural = "Bestellungskosten"
    extra = 0

    fieldsets = [
        (None, {'fields': ['kosten_name', 'bemerkung',
                           'kostenpreis', 'rabatt', 'mwstsatz', 'zwischensumme']})
    ]

    readonly_fields = ('zwischensumme', 'mwstsatz', 'kosten_name',)

    def get_additional_readonly_fields(self, request, obj=None):
        fields = ["kostenpreis"]
        if obj and obj.bezahlt:
            fields += ["rabatt"]
        return fields

    # Permissions

    NO_ADD = True

    def has_delete_permission(self, request, obj=None):
        return False if (obj and obj.bezahlt) else super().has_delete_permission(request, obj)


class BestellungInlineBestellungskostenAdd(CustomTabularInline):
    model = Bestellung.kosten.through
    verbose_name = "Bestellungskosten"
    verbose_name_plural = "Bestellungskosten hinzufügen"
    extra = 0

    autocomplete_fields = ("kosten",)

    fieldsets = [
        (None, {'fields': ['kosten', 'bemerkung', 'rabatt']})
    ]

    # Permissions

    NO_CHANGE = True
    NO_DELETE = True
    NO_VIEW = True

    def has_add_permission(self, request, obj=None):
        return False if (obj and obj.bezahlt) else super().has_add_permission(request, obj)



@admin.register(Bestellung)
class BestellungsAdmin(CustomModelAdmin):
    list_display = ('id', 'datum', 'kunde', 'status', 'zahlungsmethode',
                    'versendet', 'bezahlt', 'fix_summe', 'html_notiz')
    list_filter = ('status', 'bezahlt', 'versendet', 'zahlungsmethode')
    search_fields = ['id', 'datum', 'notiz__name', 'notiz__beschrieb', 'kundennotiz',
                     'trackingnummer'] + constants.RECHNUNGSADRESSE_FIELDS + constants.LIEFERADRESSE_FIELDS

    ordering = ("versendet", "bezahlt", "-datum")

    inlines = [BestellungInlineBestellungsposten, BestellungInlineBestellungspostenAdd,
               BestellungInlineBestellungskosten, BestellungInlineBestellungskostenAdd]

    autocomplete_fields = ("kunde", "zahlungsempfaenger", "ansprechpartner", )

    save_on_top = True

    list_select_related = ["kunde", "notiz"]

    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ('Einstellungen', {'fields': [
                    'zahlungsempfaenger', 'ansprechpartner']}),
                ('Infos', {'fields': ['name', 'datum', 'status']}),
                ('Rechnungsoptionen', {'fields': [
                    'rechnungstitel', 'rechnungstext', 'rechnungsdatum', 'zahlungskonditionen']}),
                ('Lieferung', {'fields': ['versendet', 'trackingnummer']}),
                ('Bezahlung', {'fields': [
                    'bezahlt', 'zahlungsmethode', 'summe', 'summe_mwst', 'summe_gesamt']}),
                ('Kunde', {'fields': ['kunde']}),
                ('Notizen & Texte', {
                    'fields': ['kundennotiz', 'html_notiz'],
                    'classes': ["collapse start-open"]}),
                ('Rechnungsadresse', {
                    'fields': constants.RECHNUNGSADRESSE_FIELDS if obj.bezahlt else constants.RECHNUNGSADRESSE_FIELDS_CATEGORIZED,
                    'classes': ["collapse default-open"]}),
                ('Lieferadresse', {
                    'fields': constants.LIEFERADRESSE_FIELDS if obj.versendet else constants.LIEFERADRESSE_FIELDS_CATEGORIZED,
                    'classes': ["collapse start-open"]})
            ]

        return [
            ('Einstellungen', {'fields': [
                'zahlungsempfaenger', 'ansprechpartner']}),
            ('Infos', {'fields': ['status']}),
            ('Rechnungsoptionen', {'fields': [
                'rechnungstitel', 'rechnungstext', 'rechnungsdatum', 'zahlungskonditionen']}),
            ('Lieferung', {
                'fields': ['trackingnummer'],
                'classes': ["collapse"]}),
            ('Bezahlung', {
                'fields': ['zahlungsmethode'],
                'classes': ["collapse start-open"]}),
            ('Kunde', {'fields': ['kunde']}),
            ('Notizen & Texte', {
                'fields': ['kundennotiz'],
                'classes': ["collapse start-open"]}),
        ]

    readonly_fields = ('html_notiz', 'name', 'trackinglink', 'summe', 'summe_mwst', 'summe_gesamt',)

    def get_additional_readonly_fields(self, request, obj=None):
        fields = []
        if obj:
            if obj.versendet:
                fields += ['versendet'] + \
                    constants.LIEFERADRESSE_FIELDS_WITHOUT_CONTACT
            if obj.bezahlt:
                fields += ['bezahlt', 'zahlungsmethode', 'rechnungsdatum', 'rechnungstitel',
                           'rechnungstext', 'zahlungskonditionen'] + constants.RECHNUNGSADRESSE_FIELDS_WITHOUT_CONTACT
            if obj.woocommerceid:
                fields += ["kundennotiz"]
        return fields

    # Actions

    @admin.action(description="Als bezahlt markieren", permissions=["change"])
    def als_bezahlt_markieren(self, request, queryset):
        successcount = 0
        errorcount = 0
        for bestellung in queryset.all():
            if bestellung.bezahlt:
                errorcount += 1
            else:
                bestellung.bezahlt = True
                if bestellung.versendet:
                    bestellung.status = "completed"
                bestellung.save()
                successcount += 1
        messages.success(request, (('{} Bestellungen' if successcount !=
                                    1 else 'Eine Bestellung') + ' als bezahlt markiert!').format(successcount))
        if errorcount:
            messages.error(request, ('Beim als bezahlt markieren von ' + ('{} Bestellungen' if errorcount !=
                                                                          1 else 'einer Bestellung') + ' ist ein Fehler aufgetreten!').format(errorcount))

    @admin.action(description="Bestellungen von WooCommerce aktualisieren", permissions=["change"])
    def wc_update(self, request, queryset):
        result = WooCommerce.order_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Bestellungen' if result[0] != 1 else 'Eine Bestellung') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Bestellungen' if result[1] != 1 else 'einer Bestellung') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))

    actions = [als_bezahlt_markieren, wc_update]

    # Save

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)

        # stock warnings
        if obj:
            for product in obj.produkte.all():
                product.show_stock_warning(request)

    def save_related(self, request, form, formsets, change):
        super().save_related(request, form, formsets, change)

        form.instance.second_save()

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/pdf/', self.admin_site.admin_view(views.bestellung_pdf_ansehen),
                 name='%s_%s_pdf' % info),
            path('<path:object_id>/email/rechnung/', self.admin_site.admin_view(views.bestellung_email_rechnung),
                 name='%s_%s_email_rechnung' % info),
            path('<path:object_id>/email/lieferung/', self.admin_site.admin_view(views.bestellung_email_lieferung),
                 name='%s_%s_email_lieferung' % info),
            path('<path:object_id>/duplizieren/', self.admin_site.admin_view(views.bestellung_duplizieren),
                 name='%s_%s_duplizieren' % info),
            path('<path:object_id>/zu-lieferung/', self.admin_site.admin_view(views.bestellung_zu_lieferung),
                 name='%s_%s_zu_lieferung' % info),
        ]
        return my_urls + urls


class KategorienAdminProduktInline(CustomStackedInline):
    model = Produkt.kategorien.through
    verbose_name = "Produkt in dieser Kategorie"
    verbose_name_plural = "Produkte in dieser Kategorie"
    extra = 0

    autocomplete_fields = ("produkt", )

    # Permissions

    NO_CHANGE = True


@admin.register(Kategorie)
class KategorienAdmin(CustomModelAdmin):
    fieldsets = [
        ('Infos', {'fields': ['name', 'beschrieb', 'bildlink']}),
        ('Übergeordnete Kategorie', {'fields': ['uebergeordnete_kategorie']})
    ]

    list_display = ('clean_name', 'clean_beschrieb',
                    'uebergeordnete_kategorie', 'bild', 'anzahl_produkte')
    search_fields = ['name', 'beschrieb']

    ordering = ("uebergeordnete_kategorie", "name")

    inlines = [KategorienAdminProduktInline]

    list_select_related = ("uebergeordnete_kategorie",)

    autocomplete_fields = ("uebergeordnete_kategorie", )

    # Actions

    @admin.action(description="Kategorien von WooCommerce aktualisieren", permissions=["change"])
    def wc_update(self, request, queryset):
        result = WooCommerce.category_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Kategorien' if result[0] != 1 else 'Eine Kategorie') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Kategorien' if result[1] != 1 else 'einer Kategorie') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))

    actions = ["wc_update"]


@admin.register(Kosten)
class KostenAdmin(CustomModelAdmin):
    list_display = ["clean_name", "preis", "mwstsatz"]

    search_fields = ('name', 'preis')

    ordering = ('preis', 'mwstsatz',)

    fieldsets = [
        (None, {"fields": ("name", "preis", "mwstsatz")})
    ]


class KundenAdminBestellungsInline(CustomTabularInline):
    model = Bestellung
    verbose_name = "Bestellung"
    verbose_name_plural = "Bestellungen"
    extra = 0

    show_change_link = True

    ordering = ('-datum', )

    fields = ('id', 'datum', 'fix_summe', 'versendet', 'bezahlt')

    # Permissions

    NO_CHANGE = True
    NO_ADD = True
    NO_DELETE = True


@admin.register(Kunde)
class KundenAdmin(CustomModelAdmin):
    def get_fieldsets(self, request, obj=None):
        default = [
            ('Infos', {'fields': [
                'vorname', 'nachname', 'firma', 'email', 'sprache']}),
            ('Rechnungsadresse', {
                'fields': constants.RECHNUNGSADRESSE_FIELDS_CATEGORIZED}),
            ('Lieferadresse', {
                'fields': constants.LIEFERADRESSE_FIELDS_CATEGORIZED,
                'classes': ["collapse start-open"]}),
        ]

        if obj:
            return default + [
                ('Diverses', {
                    'fields': [
                        'webseite', 'bemerkung', 'html_notiz'
                    ]}),
                ('Erweitert', {
                    'fields': [
                        'zusammenfuegen'
                    ], 'classes': ["collapse"]})
            ]

        return default + [
            ('Diverses', {'fields': ['webseite', 'bemerkung']})
        ]

    ordering = ('rechnungsadresse_plz', 'firma', 'nachname', 'vorname')

    list_display = ('id', 'firma', 'nachname', 'vorname', 'rechnungsadresse_plz',
                    'rechnungsadresse_ort', 'email', 'avatar', 'html_notiz')
    search_fields = ['id', 'nachname', 'vorname', 'firma', 'email', 'benutzername', 'webseite',
                     'notiz__name', 'notiz__beschrieb'] + constants.RECHNUNGSADRESSE_FIELDS + constants.LIEFERADRESSE_FIELDS

    readonly_fields = ["html_notiz"]

    list_select_related = ["notiz"]

    autocomplete_fields = ["zusammenfuegen"]

    inlines = [KundenAdminBestellungsInline]

    save_on_top = True

    # Actions

    @admin.action(description="Kunden von WooCommerce aktualisieren", permissions=["change"])
    def wc_update(self, request, queryset):
        result = WooCommerce.customer_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Kunden' if result[0] != 1 else 'Ein Kunde') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Kunden' if result[1] != 1 else 'einem Kunden') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))

    actions = ["wc_update"]

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/email/registriert/', self.admin_site.admin_view(views.kunde_email_registriert),
                 name='%s_%s_email_registriert' % info),
        ]
        return my_urls + urls


@admin.register(Lieferant)
class LieferantenAdmin(CustomModelAdmin):
    fieldsets = [
        ('Infos', {'fields': ['kuerzel', 'name']}),
        ('Firma', {'fields': ['webseite', 'telefon', 'email']}),
        ('Texte', {
            'fields': ['adresse', 'notiz'],
            'classes': ["collapse"]}),
        ('Ansprechpartner', {
            'fields': [
                'ansprechpartner', 'ansprechpartnertel', 'ansprechpartnermail'
            ], 'classes': ["collapse"]})
    ]

    ordering = ('kuerzel',)

    list_display = ('kuerzel', 'name', 'notiz')
    search_fields = ['kuerzel', 'name', 'adresse', 'notiz']

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/zuordnen/', self.admin_site.admin_view(views.lieferant_zuordnen),
                 name='%s_%s_zuordnen' % info),
        ]
        return my_urls + urls


class LieferungInlineProdukte(CustomTabularInline):
    model = Lieferung.produkte.through
    verbose_name = "Produkt"
    verbose_name_plural = "Produkte"
    extra = 0

    readonly_fields = ("produkt",)

    fields = ("produkt", "menge",)

    # Permissions

    NO_ADD = True

    def has_change_permission(self, request, obj=None):
        return False if obj and obj.eingelagert else super().has_change_permission(request, obj)

    def has_delete_permission(self, request, obj=None):
        return False if obj and obj.eingelagert else super().has_delete_permission(request, obj)


class LieferungInlineProdukteAdd(CustomTabularInline):
    model = Lieferung.produkte.through
    verbose_name = "Produkt"
    verbose_name_plural = "Produkte hinzufügen"
    extra = 0

    autocomplete_fields = ("produkt",)

    fields = ("produkt", "menge",)

    # Permissions

    NO_VIEW = True
    NO_CHANGE = True
    NO_DELETE = True

    def has_add_permission(self, request, obj=None):
        return False if obj and obj.eingelagert else super().has_add_permission(request, obj)

@admin.register(Lieferung)
class LieferungenAdmin(CustomModelAdmin):
    list_display = ('name', 'datum', 'anzahlprodukte',
                    'lieferant', 'eingelagert', 'html_notiz')
    list_filter = ("eingelagert", "lieferant", )

    search_fields = ["name", "datum", "lieferant",
                     "notiz__name", "notiz__beschrieb"]

    readonly_fields = ["html_notiz"]

    autocomplete_fields = ("lieferant", )

    ordering = ('-datum',)

    fieldsets = [
        ('Infos', {'fields': ['name', 'html_notiz']}),
        ('Lieferant', {'fields': ['lieferant']})
    ]

    inlines = [LieferungInlineProdukte, LieferungInlineProdukteAdd]

    save_on_top = True

    # Actions

    @admin.action(description="Lieferungen einlagern", permissions=["change"])
    def einlagern(self, request, queryset):
        successcount = 0
        errorcount = 0
        for lieferung in queryset.all():
            if lieferung.einlagern():
                successcount += 1
            else:
                errorcount += 1
        messages.success(request, (('{} Lieferungen' if successcount !=
                                    1 else 'Eine Lieferung') + ' wurde eingelagert!').format(successcount))
        if errorcount:
            messages.error(request, ('Beim Einlagern von ' + ('{} Lieferungen' if errorcount !=
                                                              1 else 'einer Lieferung') + ' ist ein Fehler aufgetreten!').format(errorcount))

    actions = ["einlagern"]

    # Views

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name

        urls = super().get_urls()

        my_urls = [
            path('<path:object_id>/einlagern/', self.admin_site.admin_view(views.lieferung_einlagern),
                 name='%s_%s_einlagern' % info),
        ]
        return my_urls + urls


@admin.register(Notiz)
class NotizenAdmin(CustomModelAdmin):
    list_display = ["name", "beschrieb", "priority", "erledigt", "erstellt_am"]
    list_filter = ["erledigt", "priority"]

    readonly_fields = ["links"]

    ordering = ["erledigt", "-priority", "erstellt_am"]

    search_fields = ("name", "beschrieb")

    def get_fieldsets(self, request, obj=None):
        if obj:
            return [
                ("Infos", {"fields": ["name", "beschrieb"]}),
                ("Daten", {"fields": ["erledigt", "priority"]}),
                ("Verknüpfungen", {"fields": ["links"]}),
            ]

        return [
            ("Infos", {"fields": ["name", "beschrieb"]}),
            ("Daten", {"fields": ["erledigt", "priority"]}),
        ]

    def get_form(self, request, obj=None, change=False, **kwargs):
        form = super().get_form(request, obj, change, **kwargs)
        if obj is None:
            form.base_fields['beschrieb'].initial = ""
            if "from_bestellung" in request.GET:
                form.base_fields['name'].initial = 'Bestellung #' + \
                    request.GET.get("from_bestellung")
                form.base_fields['beschrieb'].initial += '\n\n[Bestellung #' + \
                    request.GET.get("from_bestellung") + "]"
            if "from_produkt" in request.GET:
                form.base_fields['name'].initial = 'Produkt #' + \
                    request.GET.get("from_produkt")
                form.base_fields['beschrieb'].initial += '\n\n[Produkt #' + \
                    request.GET.get("from_produkt") + "]"
            if "from_kunde" in request.GET:
                form.base_fields['name'].initial = 'Kunde #' + \
                    request.GET.get("from_kunde")
                form.base_fields['beschrieb'].initial += '\n\n[Kunde #' + \
                    request.GET.get("from_kunde") + "]"
            if "from_lieferung" in request.GET:
                form.base_fields['name'].initial = 'Lieferung #' + \
                    request.GET.get("from_lieferung")
                form.base_fields['beschrieb'].initial += '\n\n[Lieferung #' + \
                    request.GET.get("from_lieferung") + "]"
        return form

    # Save

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if not change:
            if "from_bestellung" in request.GET:
                if Bestellung.objects.filter(pk=request.GET["from_bestellung"]).exists():
                    bestellung = Bestellung.objects.get(
                        pk=request.GET["from_bestellung"])
                    obj.bestellung = bestellung
                    obj.save()
                    messages.info(
                        request, "Bestellung #" + str(bestellung.pk) + " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Bestellung #" + request.GET["from_bestellung"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_produkt" in request.GET:
                if Produkt.objects.filter(pk=request.GET["from_produkt"]).exists():
                    produkt = Produkt.objects.get(
                        pk=request.GET["from_produkt"])
                    obj.produkt = produkt
                    obj.save()
                    messages.info(
                        request, "Produkt #" + str(produkt.pk) + " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Produkt #" + request.GET["from_produkt"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_kunde" in request.GET:
                if Kunde.objects.filter(pk=request.GET["from_kunde"]).exists():
                    kunde = Kunde.objects.get(pk=request.GET["from_kunde"])
                    obj.kunde = kunde
                    obj.save()
                    messages.info(request, "Kunde #" + str(kunde.pk) +
                                  " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Kunde #" + request.GET["from_kunde"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")
            if "from_lieferung" in request.GET:
                if Lieferung.objects.filter(pk=request.GET["from_lieferung"]).exists():
                    lieferung = Lieferung.objects.get(
                        pk=request.GET["from_lieferung"])
                    obj.lieferung = lieferung
                    obj.save()
                    messages.info(request, "Lieferung #" + str(lieferung.pk) +
                                  " wurde mit dieser Notiz verknüpft.")
                else:
                    messages.warning(
                        request, "Lieferung #" + request.GET["from_lieferung"] + " konnte nicht gefunden werden. Die Notiz wurde trotzdem erstellt.")


class ProduktInlineKategorienInline(CustomTabularInline):
    model = Produkt.kategorien.through
    extra = 0

    autocomplete_fields = ("kategorie", )

    # Permissions

    NO_CHANGE = True


@admin.register(Produkt)
class ProduktAdmin(CustomModelAdmin):
    def get_fieldsets(self, request, obj=None):
        return [
            ('Infos', {'fields': ['artikelnummer', 'name']}),
            ('Beschrieb', {
                'fields': ['kurzbeschrieb', 'beschrieb'],
                'classes': ["collapse start-open"]}),
            ('Daten', {'fields': [
                'mengenbezeichnung', 'verkaufspreis', 'mwstsatz', 'lagerbestand', 'soll_lagerbestand']}),
            ('Lieferant', {
                'fields': [
                    'lieferant', 'lieferant_preis', 'lieferant_artikelnummer', 'lieferant_url'
                ], 'classes': ["collapse start-open"]}),
            ('Aktion', {
                'fields': [
                    'aktion_von', 'aktion_bis', 'aktion_preis'
                ], 'classes': ["collapse start-open"]}),
            ('Links', {
                'fields': [
                    'datenblattlink', 'bildlink'
                ], 'classes': ["collapse"]}),
            ('Bemerkung / Notiz', {
                'fields': [
                    'bemerkung', 'html_notiz'] if obj else ['bemerkung'],
                'classes': ["collapse start-open"]})
        ]

    ordering = ('artikelnummer', 'name')

    list_display = ('artikelnummer', 'clean_name', 'clean_kurzbeschrieb',
                    'clean_beschrieb', 'preis', 'in_aktion', 'lagerbestand', 'bild', 'html_notiz')
    list_display_links = ('artikelnummer', 'in_aktion',)
    list_filter = ('lieferant', 'kategorien', 'lagerbestand')
    search_fields = ['artikelnummer', 'name', 'kurzbeschrieb',
                     'beschrieb', 'bemerkung', 'notiz__name', 'notiz__beschrieb']

    readonly_fields = ["html_notiz"]

    autocomplete_fields = ("lieferant", )

    inlines = (ProduktInlineKategorienInline,)

    save_on_top = True

    list_select_related = ["notiz"]

    # Actions

    @admin.action(description="Produkte von WooCommerce aktualisieren", permissions=["change"])
    def wc_update(self, request, queryset):
        result = WooCommerce.product_bulk_update(queryset.all())
        messages.success(request, ((
            '{} Produkte' if result[0] != 1 else 'Ein Produkt') + ' von WooCommerce aktualisiert!').format(result[0]))
        if result[1]:
            messages.error(request, ('Beim Import von ' + (
                '{} Produkten' if result[1] != 1 else 'einem Produkt') + ' von WooCommerce ist ein Fehler aufgetreten!').format(result[1]))

    @admin.action(description="Lagerbestand zurücksetzen", permissions=["change"])
    def lagerbestand_zuruecksetzen(self, request, queryset):
        for produkt in queryset.all():
            produkt.lagerbestand = 0
            produkt.save()
        messages.success(request, (('Lagerbestand von {} Produkten' if queryset.count(
        ) != 1 else 'Lagerbestand von einem Produkt') + ' zurückgesetzt!').format(queryset.count()))

    @admin.action(description="Aktion beenden", permissions=["change"])
    def aktion_beenden(self, request, queryset):
        for produkt in queryset.all():
            produkt.aktion_bis = datetime.now(utc)
            produkt.save()
        messages.success(request, (('Aktion von {} Produkten' if queryset.count(
        ) != 1 else 'Aktion von einem Produkt') + ' beendet!').format(queryset.count()))

    actions = ["wc_update", "lagerbestand_zuruecksetzen", "aktion_beenden"]

    # Save

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if obj:
            obj.show_stock_warning(request)


@admin.register(Zahlungsempfaenger)
class ZahlungsempfaengerAdmin(CustomModelAdmin):
    fieldsets = [
        ("Infos", {"fields": [
            "qriban", "logourl", "firmenname", "firmenuid"]}),
        ("Adresse", {"fields": [
            "adresszeile1", "adresszeile2", "land"]}),
        ("Daten", {"fields": [
            "email", "telefon", "webseite"]})
    ]

    list_display = ('firmenname', 'firmenuid', 'qriban')
    list_filter = ('land',)
    search_fields = ['firmenname', 'adresszeile1',
                     'adresszeile2', 'qriban', 'firmenuid']

    save_on_top = True

    # Save

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        if obj and not obj.has_valid_qr_iban():
            messages.error(request, "Ungültige QR-IBAN!")
        if obj and not obj.has_valid_uid():
            messages.warning(request, "Ungültige UID!")


# Einstellungen

@admin.register(Einstellung)
class EinstellungenAdmin(CustomModelAdmin):
    list_display = ('name', 'inhalt')
    ordering = ('name',)

    search_fields = ['name', 'description', 'char',
                     'text', 'inte', 'floa', 'url', 'email']

    readonly_fields = ["id", "name", "description"]

    hidden = True

    def get_fields(self, request, obj=None):
        return ['description'] + (
            ['char'] if obj.typ == 'char' else
            ['text'] if obj.typ == 'text' else
            ['boo'] if obj.typ == 'bool' else
            ['inte'] if obj.typ == 'int' else
            ['floa'] if obj.typ == 'float' else
            ['url'] if obj.typ == 'url' else
            ['email'] if obj.typ == 'email' else
            ['json'] if obj.typ == 'json' else []
        )

    # Permissions

    NO_ADD = True
    NO_DELETE = True


#

modeladmins = [
    (Ansprechpartner, AnsprechpartnerAdmin),
    (Bestellung, BestellungsAdmin),
    (Kategorie, KategorienAdmin),
    (Kosten, KostenAdmin),
    (Kunde, KundenAdmin),
    (Lieferant, LieferantenAdmin),
    (Lieferung, LieferungenAdmin),
    (Notiz, NotizenAdmin),
    (Produkt, ProduktAdmin),
    (Zahlungsempfaenger, ZahlungsempfaengerAdmin),
    (Einstellung, EinstellungenAdmin),
]



from django.contrib import admin

from kmuhelper.api.models import ApiKey
from kmuhelper.overrides import CustomModelAdmin


@admin.register(ApiKey)
class ApiKeyAdmin(CustomModelAdmin):
    list_display = ['id', 'name', 'user', 'read', 'write', 'key_preview']
    list_filter = ["read", "write"]

    readonly_fields = ('key', )

    fieldsets = [
        ('Settings', {'fields': ('name', 'user', ('read', 'write'))}),
        ('Key', {'fields': ('key',), "classes": ('collapse',)})
    ]

    ordering = ('id', )

    hidden = True

#


modeladmins = [
    (ApiKey, ApiKeyAdmin),
]



from defusedxml.ElementTree import parse as parse_xml, ParseError

from rich import print

from django import forms
from django.contrib import messages
from django.core.validators import FileExtensionValidator
from django.shortcuts import redirect, reverse

from kmuhelper.integrations.paymentimport.models import PaymentImport, PaymentImportEntry


def log(string, *args):
    print("[deep_pink4][KMUHelper Paymentimport][/] -", string, *args)

#####


class CamtUploadForm(forms.Form):
    """Form for uploading and processing camt.054.001.02 files"""

    file = forms.FileField(
        label="XML-Datei",
        help_text="Die Datei muss nach dem camt.053.001.04 Standard aufgebaut sein und "
                  "Detailinformationen enthalten.",
        validators=[FileExtensionValidator(['xml'])],
    )

    def clean_file(self):
        file = self.cleaned_data.get("file")

        if file:
            try:
                tree = parse_xml(file)
                root = tree.getroot()
                msg = root.find('{*}BkToCstmrStmt')

                if 'camt.053.001.04' not in str(root.tag) or msg is None:
                    raise forms.ValidationError(
                        "Die Datei liegt nicht im korrekten Format vor!"
                    )

                self.cleaned_data["camt.053.001.04"] = msg
            except ParseError as error:
                raise forms.ValidationError(
                    f"Die Datei kann nicht verarbeitet werden! {error}"
                )

        return file

    def create_payment_import_and_redirect(self, request):
        msg = self.cleaned_data.get('camt.053.001.04')
        try:
            msgid = msg.find('./{*}GrpHdr/{*}MsgId')
            msgid = msgid.text if msgid is not None else ""

            if msgid and PaymentImport.objects.filter(data_msgid=msgid).exists():
                dbentry = PaymentImport.objects.get(data_msgid=msgid)
                messages.error(
                    request,
                    f"Eine Datei mit der Nachrichtenid '{msgid}' wurde bereits "
                    "zu einem früheren Zeitpunkt importiert! "
                    "Der Import wurde abgebrochen!")
                return redirect(reverse('admin:kmuhelper_paymentimport_change', args=[dbentry.pk]))

            creationdate = msg.find('./{*}GrpHdr/{*}CreDtTm').text
            log(
                f"Start processing - Message id: '{msgid}' - Creation date: '{creationdate}'")

            dbentry = PaymentImport.objects.create(
                data_msgid=msgid,
                data_creationdate=creationdate,
            )

            for entry in msg.findall('./{*}Stmt/{*}Ntry'):
                description = entry.find('{*}AddtlNtryInf').text
                iscredit = entry.find('{*}CdtDbtInd').text == 'CRDT'

                valuedate = entry.find('./{*}ValDt/{*}Dt')
                valuedate = valuedate.text if valuedate is not None else None

                log(f"- Entry {iscredit} Description: '{description}' Value: {valuedate}")
                if iscredit:
                    if description in ['Sammelbuchung QR-Rechnung mit QR-Referenz']:
                        for txdtls in entry.findall('./{*}NtryDtls/{*}TxDtls'):
                            _amt = txdtls.find('{*}Amt')
                            amount = _amt.text
                            currency = _amt.attrib.get('Ccy', 'CHF')

                            name = txdtls.find(
                                './{*}RltdPties/{*}Dbtr/{*}Nm')
                            name = name.text if name is not None else ""
                            iban = txdtls.find(
                                './{*}RltdPties/{*}DbtrAcct/{*}Id/{*}IBAN')
                            iban = iban.text if iban is not None else ""
                            ref = txdtls.find(
                                './{*}RmtInf/{*}Strd/{*}CdtrRefInf/{*}Ref')
                            ref = ref.text if ref is not None else ""
                            addref = txdtls.find(
                                './{*}RmtInf/{*}Strd/{*}AddtlRmtInf')
                            addref = addref.text if addref is not None else ""

                            log(f"- - {amount} {currency} '{name}' '{iban}' {ref} '{addref}'")

                            PaymentImportEntry.objects.create(
                                parent=dbentry,
                                name=name,
                                iban=iban,
                                ref=ref,
                                additionalref=addref,
                                valuedate=valuedate,
                                amount=float(amount),
                                currency=currency,
                            )

            messages.success(
                request, "Zahlungsimport wurde erfolgreich importiert!"
            )
            return redirect(reverse('admin:kmuhelper_paymentimport_process', args=[dbentry.pk]))
        except AttributeError as error:
            log(error)
            messages.error(
                request, f"Bei der Verarbeitung ist ein Fehler aufgetreten: {error}")
            return redirect(reverse('admin:kmuhelper_paymentimport_upload'))



"""Methods to get settings form different places"""

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from kmuhelper.main import models

DEBUG = settings.DEBUG
AUTH_USER_MODEL = settings.AUTH_USER_MODEL


# Database settings overview

SETTINGS = {
    "wc-url": {
        "typ": "url",
        "name": "WooCommerce Shop-Url",
        "description": "Falls vorhanden: Die URL der Wordpress-Seite."
    },
    "email-stock-warning-receiver": {
        "typ": "email",
        "name": "E-Mail-Adresse für Warnungen zum Lagerbestand",
        "description": "E-Mail-Adresse für E-Mails zu durch " +
                       "Bestellungsimport hervorgerufenem, geringem Lagerbestand.\n\n" +
                       "Leer lassen, um diese E-Mails zu deaktivieren."
    },
    "email-show-buttons": {
        "typ": "bool",
        "name": "E-Mail-Knöpfe anzeigen",
        "description": "Aktivieren oder Deaktivieren der E-Mail-Knöpfe bei Bestellungen und Kunden."
    },
    "email-signature": {
        "typ": "text",
        "name": "E-Mail-Signatur",
        "description": "Standardsignatur bei E-Mails"
    },
    "default-payment-conditions": {
        "typ": "char",
        "name": "Standardzahlungskonditionen",
        "description": "Standardwert für neue Bestellungen.\n" +
                       "Skonto und Zahlungsfrist nach Syntaxdefinition von Swico.\n\n" +
                       "Beispiel: '2:15;0:30' steht für 2% Skonto bei Zahlung innerhalb " +
                       "von 15 Tagen und eine Zahlungsfrist von 30 Tagen.\n\n" +
                       "Eine Zahlungsfrist MUSS vorhanden sein und am Schluss aufgeführt werden.\n\n" +
                       "Wenn leer gelassen, wird '0:30' als Standard verwendet."
    },
    "print-payment-conditions": {
        "typ": "bool",
        "name": "Zahlungskonditionen drucken",
        "description": "Wenn aktiviert, werden die Zahlungskonditionen bei Rechnungen " +
                       "in menschenlesbarer Form unter dem Rechnungstotal gedruckt.\n\n" + 
                       "Die Zahlungskonditionen werden unabhängig von dieser Einstellung " + 
                       "immer auch in manschinenlesbarer Form im QR-Code integriert."
    },
}

SECRET_SETTINGS = {
    "wc-consumer_key": {"typ": "char"},
    "wc-consumer_secret": {"typ": "char"},
    "wc-url": {"typ": "url"},
}


def setup_settings():
    """Setup the database settings"""

    for setting in SETTINGS:
        models.Einstellung.objects.update_or_create(
            id=setting, defaults=SETTINGS[setting])

    for secretsetting in SECRET_SETTINGS:
        models.Geheime_Einstellung.objects.update_or_create(
            id=secretsetting, defaults=SECRET_SETTINGS[secretsetting])


# File settings

def get_file_setting(name, default=None):
    """Get a setting from the settings.py file"""

    return getattr(settings, name, default)


def has_file_setting(name):
    """Check if a setting in the settings.py file has been set"""

    return hasattr(settings, name)

# Get db


def get_db_setting(settingid, default=None):
    """Get a setting from the 'Einstellung' model"""

    try:
        setting = models.Einstellung.objects.get(id=settingid)
        if setting.typ in ['char', 'text'] and setting.inhalt == "":
            return default
        return setting.inhalt
    except ObjectDoesNotExist:
        return default


def get_secret_db_setting(settingid, default=None):
    """Get a setting from the 'Geheime_Einstellung' model"""

    try:
        return models.Geheime_Einstellung.objects.get(id=settingid).inhalt
    except ObjectDoesNotExist:
        return default


# Set db

def set_db_setting(settingid, content):
    """Update a database setting"""

    try:
        obj = models.Einstellung.objects.get(id=settingid)
        obj.inhalt = content
        obj.save()
        return True
    except ObjectDoesNotExist:
        return False


def set_secret_db_setting(settingid, content):
    """Update a secret database setting"""

    try:
        obj = models.Geheime_Einstellung.objects.get(id=settingid)
        obj.inhalt = content
        obj.save()
        return True
    except ObjectDoesNotExist:
        return False



# Generated by Django 3.2 on 2021-10-05 21:03

import django.core.validators
from django.db import migrations, models
import kmuhelper.main.models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0074_alter_bestellung_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='einstellung',
            name='description',
            field=models.TextField(blank=True, default='', verbose_name='Beschreibung'),
        ),
    ]



import sys
import django
from django.test.runner import DiscoverRunner
from django.conf import settings

# Create your tests here.

settings.configure(
    DEBUG=True,
    SECRET_KEY="fa^c^jx6$epw$%#b3bc)=cn_dp@misn+5l-cogxw$w3lsdl3r=",
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'testdatabase',
        }
    },
    ROOT_URLCONF='kmuhelper.urls',
    MIDDLEWARE=[
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.middleware.locale.LocaleMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    ],
    INSTALLED_APPS=(
        'kmuhelper',
        'multi_email_field',

        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
    ),
    TEMPLATES=[
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.template.context_processors.debug',
                    'django.template.context_processors.request',
                    'django.contrib.auth.context_processors.auth',
                    'django.contrib.messages.context_processors.messages',
                    'social_django.context_processors.backends',
                    'social_django.context_processors.login_redirect',
                ],
            },
        },
    ],
    KMUHELPER_DOMAIN="",
    KMUHELPER_LOG_EMAIL="",
    KMUHELPER_EMAILS_DEFAULT_CONTEXT={
        "precontent": "",
        "postcontent": "",
        "header_title": "KMUHelper TESTS",
    },
)

django.setup()
test_runner = DiscoverRunner(verbosity=1)

failures = test_runner.run_tests(['kmuhelper'])
if failures:
    sys.exit(failures)



from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.urls import reverse, reverse_lazy
from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe

#####


@login_required(login_url=reverse_lazy("admin:login"))
def app_main(request):
    return render(request, "kmuhelper/app/main.html", {})


@allow_iframe
@login_required(login_url=reverse_lazy("admin:login"))
def app_home(request):
    return render(request, "kmuhelper/app/home.html", {
        "has_permission": True,
    })


@allow_iframe
def app_error(request):
    return render(request, "kmuhelper/app/error.html", {
        "has_permission": True,
    })


@allow_iframe
@login_required(login_url=reverse_lazy("admin:login"))
def app_index(request):
    return render(request, 'admin/kmuhelper/_special/app/app_index.html', {
        'app_label': 'kmuhelper',
        'app_list': [{
            'app_label': 'kmuhelper',
            'app_url': reverse("kmuhelper:app-index"),
            'has_module_perms': True,
            'models': [
                {
                    'add_url': reverse("admin:kmuhelper_todonotiz_add"),
                    'admin_url': reverse("admin:kmuhelper_todonotiz_changelist"),
                    'name': 'ToDo-Liste',
                    'object_name': 'Notiz',
                    'perms': {
                        'add': request.user.has_perm("kmuhelper.add_todonotiz"),
                        'change': request.user.has_perm("kmuhelper.change_todonotiz"),
                        'delete': False,
                        'view': request.user.has_perm("kmuhelper.view_todonotiz")},
                    'view_only': False
                },
                {
                    'add_url': reverse("admin:kmuhelper_todoversand_add"),
                    'admin_url': reverse("admin:kmuhelper_todoversand_changelist"),
                    'name': 'Versand',
                    'object_name': 'Bestellung',
                    'perms': {
                        'add': request.user.has_perm("kmuhelper.add_todoversand"),
                        'change': request.user.has_perm("kmuhelper.change_todoversand"),
                        'delete': False,
                        'view': request.user.has_perm("kmuhelper.view_todoversand")},
                    'view_only': False
                },
                {
                    'add_url': reverse("admin:kmuhelper_todozahlungseingang_add"),
                    'admin_url': reverse("admin:kmuhelper_todozahlungseingang_changelist"),
                    'name': 'Zahlungseingang',
                    'object_name': 'Bestellung',
                    'perms': {
                        'add': request.user.has_perm("kmuhelper.add_todozahlungseingang"),
                        'change': request.user.has_perm("kmuhelper.change_todozahlungseingang"),
                        'delete': False,
                        'view': request.user.has_perm("kmuhelper.view_todozahlungseingang")},
                    'view_only': False
                },
                {
                    'add_url': reverse("admin:kmuhelper_todolagerbestand_add"),
                    'admin_url': reverse("admin:kmuhelper_todolagerbestand_changelist"),
                    'name': 'Lagerbestand',
                    'object_name': 'Produkt',
                    'perms': {
                        'add': request.user.has_perm("kmuhelper.add_todolagerbestand"),
                        'change': request.user.has_perm("kmuhelper.change_todolagerbestand"),
                        'delete': False,
                        'view': request.user.has_perm("kmuhelper.view_todolagerbestand")},
                    'view_only': False
                },
                {
                    'add_url': reverse("admin:kmuhelper_todolieferung_add"),
                    'admin_url': reverse("admin:kmuhelper_todolieferung_changelist"),
                    'name': 'Wareneingang',
                    'object_name': 'Lieferung',
                    'perms': {
                        'add': request.user.has_perm("kmuhelper.add_todolieferung"),
                        'change': request.user.has_perm("kmuhelper.change_todolieferung"),
                        'delete': False,
                        'view': request.user.has_perm("kmuhelper.view_todolieferung")},
                    'view_only': False
                },
            ],
            'name': 'KMUHelper App'
        }],
        'has_permission': True,
        'is_nav_sidebar_enabled': False,
        'is_popup': False,
        'title': 'KMUHelper App',
    })


def app_manifest(request):
    response = render(request, "kmuhelper/app/manifest.webmanifest", {})
    response['Content-Type'] = 'text/json'
    response["Service-Worker-Allowed"] = "/"
    return response



# Generated by Django 3.1 on 2020-10-05 11:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0035_auto_20201003_2125'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellungskosten',
            name='kostenpreis',
            field=models.FloatField(default=0.0, verbose_name='Preis (exkl. MwSt)'),
        ),
        migrations.AlterField(
            model_name='kosten',
            name='mwstsatz',
            field=models.FloatField(choices=[(0.0, '0.0% (Mehrwertsteuerfrei)'), (7.7, '7.7% (Normalsatz)'), (3.7, '3.7% (Sondersatz für Beherbergungsdienstleistungen)'), (2.5, '2.5% (Reduzierter Satz)')], default=7.7, verbose_name='MwSt-Satz'),
        ),
        migrations.AlterField(
            model_name='kosten',
            name='preis',
            field=models.FloatField(default=0.0, verbose_name='Preis (exkl. MwSt)'),
        ),
    ]



from django.urls import path

from kmuhelper.stats import views

#######################

urlpatterns = [
    path('', views.stats, name='stats'),

    path('products-price', views.stats_products_price,
         name='stats-products-price'),
    path('best-products', views.best_products,
         name='stats-best-products'),
]



# Generated by Django 3.2.8 on 2021-10-07 15:27

import django.core.validators
from django.db import migrations, models
import kmuhelper.main.models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0076_alter_bestellung_zahlungskonditionen'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default=kmuhelper.main.models.defaultzahlungskonditionen, help_text="Skonto und Zahlungsfrist nach Syntaxdefinition von Swico.\n\nBeispiel: '2:15;0:30' steht für 2% Skonto bei Zahlung innerhalb von 15 Tagen und eine Zahlungsfrist von 30 Tagen.\n\nEine Zahlungsfrist MUSS vorhanden sein und am Schluss aufgeführt werden.", max_length=16, validators=[django.core.validators.RegexValidator('^([0-9]+(\\.[0-9]+)?:[0-9]+;)*0:[0-9]+$', "Bitte benutze folgendes Format: 'p:d;p:d' - p = Skonto in %; d = Tage")], verbose_name='Zahlungskonditionen'),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-10 15:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0043_auto_20201010_1739'),
    ]

    operations = [
        migrations.AlterField(
            model_name='einstellung',
            name='boo',
            field=models.BooleanField(blank=True, default=False, verbose_name='Inhalt (Wahrheitswert)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='boo',
            field=models.BooleanField(blank=True, default=False, verbose_name='Inhalt (Wahrheitswert)'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-16 13:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0022_auto_20200716_1534'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ToDoEntry',
            new_name='ToDoNotiz',
        ),
        migrations.AlterModelOptions(
            name='todonotiz',
            options={'verbose_name': 'ToDo Notiz', 'verbose_name_plural': 'ToDo Notiz'},
        ),
    ]



from django.contrib.auth.decorators import login_required, permission_required
from django.urls import NoReverseMatch
from django.shortcuts import render
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.template.loader import get_template
from django.urls import reverse_lazy

from kmuhelper.utils import render_error

# Create your views here.


@login_required(login_url=reverse_lazy("admin:login"))
def home(request):
    return render(request, "kmuhelper/home.html", {"has_permission": True})


def error(request):
    return render_error(request)


@permission_required("is_superuser")
def _templatetest(request, templatename):
    """DEBUG: Test a template with given parameters"""

    try:
        get_template(templatename)
        return render(request, templatename, request.GET.dict())
    except TemplateDoesNotExist:
        return render_error(request, status=404, 
                            message=f"Vorlage '{templatename}' wurde nicht gefunden.")
    except TemplateSyntaxError as error:
        return render_error(request, status=400, 
                            message=f"Vorlage '{templatename}' enthält ungültige Syntax: {error}")
    except (NoReverseMatch) as error:
        return render_error(request, status=400, 
                            message=f"Er ist ein Fehler aufgetreten: {error}")



from django.contrib import admin
from django.db import models

from kmuhelper.main.models import Notiz, Bestellung, Produkt, Lieferung

#####


class ToDoNotizManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(erledigt=False)


class ToDoNotiz(Notiz):
    objects = ToDoNotizManager()

    class Meta:
        proxy = True
        verbose_name = "Notiz"
        verbose_name_plural = "Notizen"
        default_permissions = ('add', 'change', 'view')


class ToDoVersandManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(versendet=False)


class ToDoVersand(Bestellung):
    objects = ToDoVersandManager()

    class Meta:
        proxy = True
        verbose_name = "Bestellung"
        verbose_name_plural = "Bestellungen"
        default_permissions = ('add', 'change', 'view')


class ToDoZahlungseingangManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(bezahlt=False)


class ToDoZahlungseingang(Bestellung):
    objects = ToDoZahlungseingangManager()

    class Meta:
        proxy = True
        verbose_name = "Bestellung"
        verbose_name_plural = "Bestellung"
        default_permissions = ('add', 'change', 'view')


class ToDoLagerbestand(Produkt):
    @admin.display(description="Preis")
    def preis(self, *args, **kwargs):
        return super().preis(*args, **kwargs)

    @admin.display(description="Nr.", ordering="artikelnummer")
    def nr(self):
        return self.artikelnummer

    class Meta:
        proxy = True
        verbose_name = "Produkt"
        verbose_name_plural = "Produkte"
        default_permissions = ('add', 'change', 'view')


class ToDoLieferungsManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(eingelagert=False)


class ToDoLieferung(Lieferung):
    objects = ToDoLieferungsManager()

    class Meta:
        proxy = True
        verbose_name = "Lieferung"
        verbose_name_plural = "Lieferungen"
        default_permissions = ('add', 'change', 'view')



# Generated by Django 3.1.7 on 2021-04-03 17:41

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0053_bestellung_rechnungstext'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellung',
            name='rechnungstitel',
            field=models.CharField(blank=True, default='', help_text="Titel der Rechnung. Leer lassen für 'RECHNUNG'", max_length=32, verbose_name='Rechnungstitel'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default='0:30', help_text="Skonto und Zahlfrist nach Syntaxdefinition von Swico. z.B. '2:15;0:30'", max_length=16, validators=[django.core.validators.RegexValidator('[0-9]+:[0-9]+(;[0-9]+:[0-9]+)*')], verbose_name='Zahlungskonditionen'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsdatum',
            field=models.DateField(blank=True, default=None, help_text='Datum der Rechnung. Wird auch als Startpunkt für die Zahlungskonditionen verwendet.', null=True, verbose_name='Rechnungsdatum'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungstext',
            field=models.TextField(blank=True, default='', help_text="Wird auf der Rechnung gedruckt! Unterstützt <abbr title='<b>Fett</b><u>Unterstrichen</u><i>Kursiv</i>'>XML markup</abbr>.", verbose_name='Rechnungstext'),
        ),
    ]



# Generated by Django 3.2 on 2021-04-16 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0067_auto_20210415_2007'),
    ]

    operations = [
        migrations.RenameField(
            model_name='kunde',
            old_name='registrierungsemail',
            new_name='email_registriert',
        ),
    ]



"""Base class for PDFCreators"""

from io import BytesIO

from reportlab.platypus import SimpleDocTemplate
from reportlab.lib.units import mm
from reportlab.lib.pagesizes import A4

from django.http import FileResponse
from django.utils import translation
_ = translation.gettext


class PDFGenerator():
    def __init__(self, *args, **kwargs):
        self.language = self.get_language(*args, **kwargs)

        self.__buffer = BytesIO()
        self.__doc = SimpleDocTemplate(
            self.__buffer, rightMargin=10*mm, leftMargin=10*mm, topMargin=10*mm, bottomMargin=10*mm,
            pagesize=A4, creator=kwargs.get('author', "KMUHelper"))

        cur_language = translation.get_language()
        translation.activate(self.language)

        self.__elements = self.get_elements(*args, **kwargs)
        self.__doc.build(self.__elements)
        self.__buffer.seek(0)

        translation.activate(cur_language)

    def get_elements(self, *args, **kwargs):
        raise NotImplementedError()

    def get_language(self, *args, **kwargs):
        print(
            f"[KMUHelper] - {self.__class__.__name__} - get_language hasn't been implemented, using default")
        return translation.get_language()

    def get_pdf(self):
        return self.__buffer

    def get_response(self, as_attachment=False, filename="document.pdf"):
        return FileResponse(self.__buffer, as_attachment=as_attachment, filename=filename)



# Generated by Django 3.2.9 on 2021-11-22 18:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0077_alter_bestellung_zahlungskonditionen'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_adresszeile1',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Adresszeile 1'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_adresszeile2',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Adresszeile 2'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_firma',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Firma'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_nachname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Nachname'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_ort',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Ort'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='lieferadresse_vorname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Vorname'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_adresszeile1',
            field=models.CharField(blank=True, default='', help_text="Strasse und Hausnummer oder 'Postfach'", max_length=250, verbose_name='Adresszeile 1'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_adresszeile2',
            field=models.CharField(blank=True, default='', help_text='Wird in QR-Rechnung NICHT verwendet!', max_length=250, verbose_name='Adresszeile 2'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_firma',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Firma'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_nachname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Nachname'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_ort',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Ort'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_vorname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Vorname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='firma',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Firma'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_adresszeile1',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Adresszeile 1'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_adresszeile2',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Adresszeile 2'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_firma',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Firma'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_nachname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Nachname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_ort',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Ort'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='lieferadresse_vorname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Vorname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='nachname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Nachname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_adresszeile1',
            field=models.CharField(blank=True, default='', help_text='Strasse und Hausnummer oder "Postfach"', max_length=250, verbose_name='Adresszeile 1'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_adresszeile2',
            field=models.CharField(blank=True, default='', help_text='Wird in QR-Rechnung NICHT verwendet!', max_length=250, verbose_name='Adresszeile 2'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_firma',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Firma'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_nachname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Nachname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_ort',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Ort'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='rechnungsadresse_vorname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Vorname'),
        ),
        migrations.AlterField(
            model_name='kunde',
            name='vorname',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Vorname'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-15 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0016_auto_20200627_1641'),
    ]

    operations = [
        migrations.CreateModel(
            name='ToDoEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('beschrieb', models.TextField(verbose_name='Beschrieb')),
                ('erledigt', models.BooleanField(default=False, verbose_name='Erledigt')),
                ('priority', models.IntegerField(default=0, verbose_name='Priorität')),
                ('erstellt_am', models.DateTimeField(auto_now_add=True, verbose_name='Erstellt am')),
            ],
            options={
                'verbose_name': 'ToDo-Eintrag',
                'verbose_name_plural': 'ToDo-Einträge',
            },
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-10 15:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0042_auto_20201010_1633'),
    ]

    operations = [
        migrations.AddField(
            model_name='einstellung',
            name='boo',
            field=models.BooleanField(blank=True, default=True, verbose_name='Inhalt (Wahrheitswert)'),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='boo',
            field=models.BooleanField(blank=True, default=True, verbose_name='Inhalt (Wahrheitswert)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('bool', 'Wahrheitswert'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail')], default='char', max_length=5, verbose_name='Typ'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('bool', 'Wahrheitswert'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail')], default='char', max_length=5, verbose_name='Typ'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-05-31 09:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0008_auto_20200502_1851'),
    ]

    operations = [
        migrations.AddField(
            model_name='kunde',
            name='notiz',
            field=models.TextField(blank=True, default='', verbose_name='Notiz'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-05-02 16:51

import django.core.validators
from django.db import migrations, models
import kmuhelper.main.models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0007_auto_20200502_1530'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='order_key',
            field=models.CharField(blank=True, default=kmuhelper.main.models.defaultorderkey, max_length=50, verbose_name='Bestellungs-Schlüssel'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='trackingnummer',
            field=models.CharField(blank=True, default='', help_text='Bitte gib hier eine Trackingnummer der Schweizer Post ein. (optional)', max_length=25, validators=[django.core.validators.RegexValidator('^99\\.[0-9]{2}\\.[0-9]{6}\\.[0-9]{8}$', 'Bite benutze folgendes Format: 99.xx.xxxxxx.xxxxxxxx')], verbose_name='Trackingnummer'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-27 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0030_auto_20200720_1936'),
    ]

    operations = [
        migrations.AlterField(
            model_name='produkt',
            name='kategorien',
            field=models.ManyToManyField(related_name='produkte', through='kmuhelper.Produktkategorie', to='kmuhelper.Kategorie', verbose_name='Kategorie'),
        ),
    ]



"""Useful constants used in multiple places"""

ORDERSTATUS = [
    ("pending", "Zahlung ausstehend"),
    ("processing", "In Bearbeitung"),
    ("on-hold", "In Wartestellung"),
    ("completed", "Abgeschlossen"),
    ("cancelled", "Storniert/Abgebrochen"),
    ("refunded", "Rückerstattet"),
    ("failed", "Fehlgeschlagen"),
    ("trash", "Gelöscht")
]

MWSTSETS = [
    (0.0, "0.0% (Mehrwertsteuerfrei)"),
    (7.7, "7.7% (Normalsatz)"),
    (3.7, "3.7% (Sondersatz für Beherbergungsdienstleistungen)"),
    (2.5, "2.5% (Reduzierter Satz)")
]

PAYMENTMETHODS = [
    ("bacs", "Überweisung"),
    ("cheque", "Scheck"),
    ("cod", "Rechnung / Nachnahme"),
    ("paypal", "PayPal")
]

COUNTRIES = [
    ("CH", "Schweiz"),
    ("LI", "Liechtenstein")
]

LANGUAGES = [
    ("de", "Deutsch [DE]"),
    ("fr", "Französisch [FR]"),
    ("it", "Italienisch [IT]"),
    ("en", "Englisch [EN]")
]

RECHNUNGSADRESSE_FIELDS_WITHOUT_CONTACT = [
    'rechnungsadresse_vorname', 'rechnungsadresse_nachname',
    'rechnungsadresse_firma',
    'rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2',
    'rechnungsadresse_plz', 'rechnungsadresse_ort',
    'rechnungsadresse_kanton', 'rechnungsadresse_land',
]

RECHNUNGSADRESSE_FIELDS = [
    'rechnungsadresse_vorname', 'rechnungsadresse_nachname',
    'rechnungsadresse_firma',
    'rechnungsadresse_adresszeile1', 'rechnungsadresse_adresszeile2',
    'rechnungsadresse_plz', 'rechnungsadresse_ort',
    'rechnungsadresse_kanton', 'rechnungsadresse_land',
    'rechnungsadresse_email', 'rechnungsadresse_telefon'
]

RECHNUNGSADRESSE_FIELDS_CATEGORIZED = [
    ('rechnungsadresse_vorname', 'rechnungsadresse_nachname'),
    'rechnungsadresse_firma',
    ('rechnungsadresse_adresszeile1',
     'rechnungsadresse_adresszeile2'),
    ('rechnungsadresse_plz', 'rechnungsadresse_ort'),
    ('rechnungsadresse_kanton', 'rechnungsadresse_land'),
    ('rechnungsadresse_email', 'rechnungsadresse_telefon')
]

LIEFERADRESSE_FIELDS_WITHOUT_CONTACT = [
    'lieferadresse_vorname', 'lieferadresse_nachname',
    'lieferadresse_firma',
    'lieferadresse_adresszeile1', 'lieferadresse_adresszeile2',
    'lieferadresse_plz', 'lieferadresse_ort',
    'lieferadresse_kanton', 'lieferadresse_land',
]

LIEFERADRESSE_FIELDS = [
    'lieferadresse_vorname', 'lieferadresse_nachname',
    'lieferadresse_firma',
    'lieferadresse_adresszeile1', 'lieferadresse_adresszeile2',
    'lieferadresse_plz', 'lieferadresse_ort',
    'lieferadresse_kanton', 'lieferadresse_land',
    'lieferadresse_email', 'lieferadresse_telefon'
]

LIEFERADRESSE_FIELDS_CATEGORIZED = [
    ('lieferadresse_vorname', 'lieferadresse_nachname'),
    'lieferadresse_firma',
    ('lieferadresse_adresszeile1',
     'lieferadresse_adresszeile2'),
    ('lieferadresse_plz', 'lieferadresse_ort'),
    ('lieferadresse_kanton', 'lieferadresse_land'),
    ('lieferadresse_email', 'lieferadresse_telefon')
]



from django.urls import path

from kmuhelper.app import views

#######################

urlpatterns = [
    path('', views.app_main, name="app-main"),
    path('home', views.app_home, name="app-home"),
    path('error', views.app_error, name="app-error"),
    path('index', views.app_index, name="app-index"),

    path('manifest.webmanifest', views.app_manifest, name="app-manifest"),
]



# Generated by Django 3.1.7 on 2021-04-03 18:08

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0054_auto_20210403_1941'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default='0:30', help_text="Skonto und Zahlfrist nach Syntaxdefinition von Swico. z.B. '2:15;0:30'", max_length=16, validators=[django.core.validators.RegexValidator('^[0-9]+:[0-9]+(;[0-9]+:[0-9]+)*$')], verbose_name='Zahlungskonditionen'),
        ),
    ]



# Generated by Django 3.2 on 2021-10-05 21:29

import django.core.validators
from django.db import migrations, models
import kmuhelper.main.models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0075_einstellung_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default=kmuhelper.main.models.defaultzahlungskonditionen, help_text="Skonto und Zahlfrist nach Syntaxdefinition von Swico. z.B. '2:15;0:30' steht für 2% Skonto bei Zahlung in 15 Tagen und eine Zahlungsfrist von 30 Tagen.", max_length=16, validators=[django.core.validators.RegexValidator('^[0-9]+:[0-9]+(;[0-9]+:[0-9]+)*$', "Bitte benutze folgendes Format: 'p:d;p:d' - p = Skonto in %; d = Tage")], verbose_name='Zahlungskonditionen'),
        ),
    ]
